@@ -1,18 +1,17 @@
 package MAB2::Parser::Disk;
 
-our $VERSION = '0.20';
+our $VERSION = '0.21';
 
 use strict;
 use warnings;
 use charnames qw< :full >;
-use Carp qw(croak);
+use Carp qw(carp croak);
 use Readonly;
 
 Readonly my $SUBFIELD_INDICATOR => qq{\N{INFORMATION SEPARATOR ONE}};
 Readonly my $END_OF_FIELD       => qq{\N{LINE FEED}};
 Readonly my $END_OF_RECORD      => q{};
 
-
 sub new {
     my $class = shift;
     my $file  = shift;
@@ -40,7 +39,6 @@ sub new {
     return ( bless $self, $class );
 }
 
-
 sub next {
     my $self = shift;
     local $/ = $END_OF_RECORD;
@@ -55,57 +53,58 @@ sub next {
     return;
 }
 
-
 sub _decode {
     my $reader = shift;
     chomp($reader);
 
     my @record;
 
-    my @fields = split($END_OF_FIELD, $reader);
+    my @fields = split( $END_OF_FIELD, $reader );
 
     my $leader = shift @fields;
-    if( $leader =~ m/^\N{NUMBER SIGN}{3}\s(\d{5}[cdnpu]M2.0\d{7}\s{6}\w)/xms ){
+    if ($leader =~ m/^\N{NUMBER SIGN}{3}\s(\d{5}[cdnpu]M2.0\d{7}\s{6}\w)/xms )
+    {
         push( @record, [ 'LDR', '', '_', $1 ] );
     }
-    else{
-        croak "record leader not valid: $leader";
+    else {
+        carp "faulty record leader: $leader";
     }
 
-    # ToDo: skip faulty fields
     foreach my $field (@fields) {
-        croak "incomplete field: \"$field\"" if length($field) <= 4;
-        my $tag = substr( $field, 0, 3 );
-        my $ind = substr( $field, 3, 1 );
-        my $data = substr( $field, 4 );
 
-        # check for a 3-digit numeric tag
-        ( $tag =~ m/^[0-9]{3}$/xms ) or croak "Invalid tag: \"$tag\"";
+        if ( length $field <= 4 ) {
+            carp "faulty field: \"$field\"";
+            next;
+        }
 
-        # check if indicator is an single alphabetic character
-        ( $ind =~ m/^[a-z\s]$/xms ) or croak "Invalid indicator: \"$ind\"";
-
-        # check if data contains subfield indicators
-        if ( $data =~ m/^\s*($SUBFIELD_INDICATOR|\$)(.*)/ ) {
-            my $subfield_indicator = $1;
-            my @subfields = split( $subfield_indicator, $2 );
-            ( @subfields ) or croak "no subfield data found: \"$tag$ind$data\"";
-            push(
-                @record,
-                [   $tag,
+        if ( my ( $tag, $ind, $data )
+            = $field =~ m/^(\d{3})([A-Za-z0-9\s])(.*)/ )
+        {
+            # check if data contains subfield indicators
+            if ( $data =~ m/\s*($SUBFIELD_INDICATOR|\$)(.*)/ ) {
+                my $subfield_indicator = $1;
+                push
+                    @record,
+                    [
+                    $tag,
                     $ind,
-                    map { substr( $_, 0, 1 ), substr( $_, 1 ) } @subfields
-                ]
-            );
+                    map { ( substr( $_, 0, 1 ), substr( $_, 1 ) ) }
+                        split /$subfield_indicator/,
+                    $1
+                    ];
+            }
+            else {
+                push @record, [ $tag, $ind, '_', $data ];
+            }
         }
         else {
-            push( @record, [ $tag, $ind, '_', $data ] );
+            carp "faulty field structure: \"$field\"";
+            next;
         }
     }
-    return \@record;    
+    return \@record;
 }
 
-
 1;    # End of MAB2::Parser::Disk
 
 __END__
@@ -122,8 +121,8 @@ MAB2::Parser::Disk - MAB2 Diskette format parser
 
 L<MAB2::Parser::Disk> is a parser for MAB2 Diskette records.
 
-L<MAB2::Parser::Disk> expects UTF-8 encoded files as input. Otherwise provide a 
-filehande with a specified I/O layer.
+L<MAB2::Parser::Disk> expects UTF-8 encoded files as input. Otherwise 
+provide a filehande with a specified I/O layer.
 
     use MAB2::Parser::Disk;
 
